==> hermes-3.0.1.26.tar.gz: hermes-3.0.1.26/app/controllers/hermes/api/dmo/nuds_controller.rb <==
# frozen_string_literal: true

# checker Api::Dmo::HermesController avant d'ajouter des méthodes
module Hermes
  module Api
    module Dmo
      class NudsController < Hermes::Api::Dmo::ApplicationController
        
        before_action :set_instance_from_unique_number, only: [:send_pin_code_on_mail, :authentify_with_pin_code, :hydrate_housing_request_from_sne, :send_new_pin_code]
        
        VALID_OPERATORS = %w[
          $in
          $nin
          $ne
          $gt
          $gte
          $lt
          $lte
          $regex
        ].freeze

        ACCEPTABLE_PARAMS = %w[
          ids
          department
          creation_date
          unique_number
          is_active
          housing_request_id
        ].freeze

        REGEX_PARAMS = %w[
        ].freeze

        MUTATE_PARAMS = %i[
          department
          creation_date
          unique_number
          is_active
          housing_request_id
        ].freeze
        
       mutate_params_only(*MUTATE_PARAMS)

        def index
          acceptable_params *ACCEPTABLE_PARAMS
          regex_params *REGEX_PARAMS
          valid_operators *VALID_OPERATORS
          super
        end

        def create
          user_id = @user.id
          param_user = params&.fetch(:data, nil)&.fetch(:attributes, nil)&.fetch(:user_id, nil)
          user_id = param_user if @user.has_role?('Administrator') && param_user
          valid_params = mutate_valid_params.merge(user_id: user_id)
          valid_params[:housing_request_id] ||= Hermes::HousingRequest.find_by(als_id: params[:data][:attributes][:als_id]).id
          if (@instance = @model.find_by(
              unique_number: valid_params[:unique_number],
              housing_request_id: valid_params[:housing_request_id]
          ))
            return render_json_error(:unprocessable_entity, :not_writers) unless @instance.reload && @instance.writable_by?(@user)
            return respond_with_errors(@instance) unless @instance.update(
                valid_params.except(
                    :unique_number,
                    :housing_request_id,
                    :user_id
                )
            )
          else
            return respond_with_errors(@instance) unless (@instance = @model.create(valid_params))
          end
          return respond_with_errors(@instance) unless @instance.save

          render(jsonapi: @instance, serializer: @serializer, status: :created)
        end
        
        def unique_number_is_in_dep_sne(unique_number)
          deps = get_dep_list("SNE") + get_dep_list("IMHOWEB")
          deps.any? { |dep| unique_number[0..2] === (dep.length ==2 ? "0"+dep : dep) }
        end
        
        def set_instance_from_unique_number
          return render_json_error(:unprocessable_entity, :unique_number_not_covered_by_sne) unless unique_number_is_in_dep_sne(params[:id])
          @instance = Hermes::HousingRequestInitializer.order_by(created_at: :desc).find_by(unique_number: params[:id])
        end

        def get_dep_list(type)
          return [] if !HERMES_CONFIG["DEP_PROVIDERS"] || HERMES_CONFIG["DEP_PROVIDERS"][type].nil?
          HERMES_CONFIG["DEP_PROVIDERS"][type]
        end

        def sne_department_list
          render body:get_dep_list("SNE"), status: 200
        end

        def imhoweb_department_list
          render body:get_dep_list("IMHOWEB"), status: 200
        end

        def pending_for_current_user
          @instance =  Hermes::HousingRequestInitializer.find_by(user: current_user, done: false, :pin_code_created_at.ne => nil)
          return render_json_error(:not_found, :instance_not_found) unless @instance

          render(
            jsonapi: @instance ,
            serializer: Hermes::HousingRequestInitializerSerializer,
            **@options
          )

        end 

        def hydrate_housing_request_from_sne
          return render_json_error(:unauthorized, :not_als_collaborator) unless current_user_roles.include?('collaborateur offre logement')
          @instance = Hermes::HousingRequestInitializer.create(unique_number: params[:id], user_id: params[:user_id]) if !@instance
          return render_json_error(:bad_request, :unique_nuber_already_exists_in_hermes) if Hermes::Nud.find_by(unique_number: params[:id])
          concerned_user = User.find_by(id: params[:user_id])
          return render_json_error(:unprocessable_entity, :unknown_user_id) if !concerned_user
          housing_request = @instance.hydrate_housing_request_from_sne(concerned_user.id)
          render(
            jsonapi:housing_request,
            serializer: Hermes::HousingRequestSerializer,
            **@options
          )
          
        end
        
        def generic_send_pin_code
          @instance.update(user: current_user, done: false)
          result = @instance.send_pin_code_on_mail(current_user)
          if result.is_a? Hash
            render json: result, status: 422
          elsif result.is_a? String
            render json: { email: @instance.hidden_email }, status: 200
          else 
            render_json_error(:unprocessable_entity, :missing_pin_code)
          end
        end 

        def send_pin_code_on_mail
          @instance =  Hermes::HousingRequestInitializer.create(unique_number: params[:id], user: current_user) if !@instance
          return render_json_error(:unprocessable_entity, :recuperation_of_housing_request_from_sne_already_done) if @instance.housing_request&.is_active
          generic_send_pin_code
        end 
          
        def send_new_pin_code
          @instance =  Hermes::HousingRequestInitializer.create(unique_number: params[:id], user: current_user) if !@instance
          generic_send_pin_code
        end

        def user_can_destroy?
          is_writer_and_is_owner = (@instance.user_id.to_s == current_user.id.to_s) &&
            (current_user_roles & ['Administrator', 'Manager', 'cre']).present?
  
          is_writer_and_is_owner || is_super_admin || @user.roles&.include?("Administrator") || @user.auth_roles&.include?('superadministrateur') || @user.auth_roles&.include?('collaborateur offre logement')
        end

        def destroy
          return render_json_error(:not_found, :instance_not_found) unless @instance
          return render_json_error(:not_found, :not_admin_or_manager) unless user_can_destroy?
          return render(json: @instance.errors, status: :unprocessable_entity) unless @instance.destroy
  
          render(body: nil, status: :no_content)
        end

        def authentify_with_pin_code
          return render_json_error(:not_found, :object_not_found) unless @instance
          return render_json_error(:unprocessable_entity, :recuperation_of_housing_request_from_sne_already_done) if @instance.housing_request&.is_active
          return render_json_error(:unprocessable_entity, :unable_to_send_mail_with_pin_code) unless @instance.check_pin_code(params["pin_code"])
          housing_request = @instance.hydrate_housing_request_from_sne(user_id = @user.id)
          housing_request_folder = housing_request.reload.housing_request_folder 
          if !Rails.env.test?
            # housing_request_folder.syncro_files_from_sne(@instance.unique_number)
          end
          
          housing_request_folder.save
          render(
            jsonapi:housing_request,
            serializer: Hermes::HousingRequestSerializer,
            **@options
          )
        end
      end
    end
  end
end

==> hermes-3.1.0.0.tar.gz: hermes-3.1.0.0/app/controllers/hermes/api/dmo/nuds_controller.rb <==
# frozen_string_literal: true

# checker Api::Dmo::HermesController avant d'ajouter des méthodes
module Hermes
  module Api
    module Dmo
      class NudsController < Hermes::Api::Dmo::ApplicationController
        
        before_action :set_instance_from_unique_number, only: [:send_pin_code_on_mail, :authentify_with_pin_code, :hydrate_housing_request_from_sne, :send_new_pin_code]
        
        VALID_OPERATORS = %w[
          $in
          $nin
          $ne
          $gt
          $gte
          $lt
          $lte
          $regex
        ].freeze

        ACCEPTABLE_PARAMS = %w[
          ids
          department
          creation_date
          unique_number
          is_active
          housing_request_id
        ].freeze

        REGEX_PARAMS = %w[
        ].freeze

        MUTATE_PARAMS = %i[
          department
          creation_date
          unique_number
          is_active
          housing_request_id
        ].freeze
        
       mutate_params_only(*MUTATE_PARAMS)

        def index
          acceptable_params *ACCEPTABLE_PARAMS
          regex_params *REGEX_PARAMS
          valid_operators *VALID_OPERATORS
          super
        end

        def create
          user_id = @user.id
          param_user = params&.fetch(:data, nil)&.fetch(:attributes, nil)&.fetch(:user_id, nil)
          user_id = param_user if @user.has_role?('Administrator') && param_user
          valid_params = mutate_valid_params.merge(user_id: user_id)
          valid_params[:housing_request_id] ||= Hermes::HousingRequest.find_by(als_id: params[:data][:attributes][:als_id]).id
          if (@instance = @model.find_by(
              unique_number: valid_params[:unique_number],
              housing_request_id: valid_params[:housing_request_id]
          ))
            return render_json_error(:unprocessable_entity, :not_writers) unless @instance.reload && @instance.writable_by?(@user)
            return respond_with_errors(@instance) unless @instance.update(
                valid_params.except(
                    :unique_number,
                    :housing_request_id,
                    :user_id
                )
            )
          else
            return respond_with_errors(@instance) unless (@instance = @model.create(valid_params))
          end
          return respond_with_errors(@instance) unless @instance.save

          render(jsonapi: @instance, serializer: @serializer, status: :created)
        end
        
        def unique_number_is_in_dep_sne(unique_number)
          deps = get_dep_list("SNE") + get_dep_list("IMHOWEB")
          deps.any? { |dep| unique_number[0..2] === (dep.length ==2 ? "0"+dep : dep) }
        end
        
        def set_instance_from_unique_number
          return render_json_error(:unprocessable_entity, :unique_number_not_covered_by_sne) unless unique_number_is_in_dep_sne(params[:id])
          @instance = Hermes::HousingRequestInitializer.order_by(created_at: :desc).find_by(unique_number: params[:id])
        end

        def get_dep_list(type)
          return [] if !HERMES_CONFIG["DEP_PROVIDERS"] || HERMES_CONFIG["DEP_PROVIDERS"][type].nil?
          HERMES_CONFIG["DEP_PROVIDERS"][type]
        end

        def sne_department_list
          render body:get_dep_list("SNE"), status: 200
        end

        def imhoweb_department_list
          render body:get_dep_list("IMHOWEB"), status: 200
        end

        def pending_for_current_user
          @instance =  Hermes::HousingRequestInitializer.find_by(user: current_user, done: false, :pin_code_created_at.ne => nil)
          return render_json_error(:not_found, :instance_not_found) unless @instance

          render(
            jsonapi: @instance ,
            serializer: Hermes::HousingRequestInitializerSerializer,
            **@options
          )

        end 

        def hydrate_housing_request_from_sne
          return render_json_error(:unauthorized, :not_als_collaborator) unless current_user_roles.include?('collaborateur offre logement')
          @instance = Hermes::HousingRequestInitializer.create(unique_number: params[:id]) if !@instance
          return render_json_error(:bad_request, :unique_nuber_already_exists_in_hermes) if Hermes::Nud.find_by(unique_number: params[:id])
          concerned_user = User.find_by(id: params[:user_id])
          return render_json_error(:unprocessable_entity, :unknown_user_id) if !concerned_user
          housing_request = @instance.hydrate_housing_request_from_sne(concerned_user.id)
          render(
            jsonapi:housing_request,
            serializer: Hermes::HousingRequestSerializer,
            **@options
          )
          
        end
        
        def generic_send_pin_code
          @instance.update(user: current_user, done: false)
          result = @instance.send_pin_code_on_mail(current_user)
          if result.is_a? Hash
            render json: result, status: 422
          elsif result.is_a? String
            render json: { email: @instance.hidden_email }, status: 200
          else 
            render_json_error(:unprocessable_entity, :missing_pin_code)
          end
        end 

        def send_pin_code_on_mail
          @instance =  Hermes::HousingRequestInitializer.create(unique_number: params[:id]) if !@instance
          return render_json_error(:unprocessable_entity, :recuperation_of_housing_request_from_sne_already_done) if @instance.cant_create_new
          generic_send_pin_code
        end 
          
        def send_new_pin_code
          @instance =  Hermes::HousingRequestInitializer.create(unique_number: params[:id]) if !@instance
          generic_send_pin_code
        end

        def user_can_destroy?
          is_writer_and_is_owner = (@instance.user_id.to_s == current_user.id.to_s) &&
            (current_user_roles & ['Administrator', 'Manager', 'cre']).present?
  
          is_writer_and_is_owner || is_super_admin || @user.roles&.include?("Administrator") || @user.auth_roles&.include?('superadministrateur') || @user.auth_roles&.include?('collaborateur offre logement')
        end

        def destroy
          return render_json_error(:not_found, :instance_not_found) unless @instance
          return render_json_error(:not_found, :not_admin_or_manager) unless user_can_destroy?
          return render(json: @instance.errors, status: :unprocessable_entity) unless @instance.destroy
  
          render(body: nil, status: :no_content)
        end

        def authentify_with_pin_code
          return render_json_error(:not_found, :object_not_found) unless @instance
          return render_json_error(:unprocessable_entity, :recuperation_of_housing_request_from_sne_already_done) if @instance.cant_create_new
          return render_json_error(:unprocessable_entity, :unable_to_send_mail_with_pin_code) unless @instance.check_pin_code(params["pin_code"])
          housing_request = @instance.hydrate_housing_request_from_sne(user_id = @user.id)
          housing_request_folder = housing_request.reload.housing_request_folder 
          if !Rails.env.test?
            # housing_request_folder.syncro_files_from_sne(@instance.unique_number)
          end
          
          housing_request_folder.save
          render(
            jsonapi:housing_request,
            serializer: Hermes::HousingRequestSerializer,
            **@options
          )
        end
      end
    end
  end
end
